@@ -130,7 +130,15 @@ class VadYaraScanner(BaseYaraScanner):
         BaseYaraScanner.__init__(self, address_space = task.get_process_address_space(), **kwargs)
 
     def scan(self, offset = 0, maxlen = None):
-        for vad, self.address_space in self.task.get_vads(skip_max_commit = True):
+    
+        if maxlen == None:
+            vads = self.task.get_vads(skip_max_commit = True)
+        else:
+            filter = lambda x : x.Length < maxlen
+            vads = self.task.get_vads(vad_filter = filter, 
+                skip_max_commit = True)
+        
+        for vad, self.address_space in vads:
             for match in BaseYaraScanner.scan(self, vad.Start, vad.Length):
                 yield match
 
@@ -196,6 +204,9 @@ class YaraScan(taskmods.DllList):
         config.add_option('REVERSE', short_option = 'R', default = 0,
                           help = 'Reverse this number of bytes',
                           action = 'store', type = 'int')
+        config.add_option('MAX-SIZE', short_option = 'M', default = 0x40000000, 
+                          action = 'store', type = 'long', 
+                          help = 'Set the maximum size (default is 1GB)') 
 
     def _compile_rules(self):
         """Compile the YARA rules from command-line parameters. 
@@ -232,7 +243,7 @@ class YaraScan(taskmods.DllList):
     def _scan_process_memory(self, addr_space, rules):
         for task in self.filter_tasks(tasks.pslist(addr_space)):
             scanner = VadYaraScanner(task = task, rules = rules)
-            for hit, address in scanner.scan():
+            for hit, address in scanner.scan(maxlen = self._config.MAX_SIZE):
                 yield (task, address, hit, scanner.address_space.zread(address - self._config.REVERSE, self._config.SIZE))
 
     def _scan_kernel_memory(self, addr_space, rules):
